/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.system.service.impl;

import com.alicp.jetcache.anno.CacheType;
import com.alicp.jetcache.anno.Cached;
import com.zayac.admin.common.constant.CacheConstants;
import com.zayac.admin.common.enums.DisEnableStatusEnum;
import com.zayac.admin.system.mapper.AccountMapper;
import com.zayac.admin.system.model.entity.AccountDO;
import com.zayac.admin.system.model.entity.UserDO;
import com.zayac.admin.system.model.query.AccountQuery;
import com.zayac.admin.system.model.req.AccountReq;
import com.zayac.admin.system.model.resp.AccountDetailResp;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.service.AccountService;
import com.zayac.admin.system.service.UserService;
import lombok.RequiredArgsConstructor;

import org.springframework.stereotype.Service;

import top.continew.starter.extension.crud.service.impl.BaseServiceImpl;

import java.util.List;

/**
 * 账号业务实现
 *
 * @author zayac
 * @since 2024/05/10 20:44
 */
@Service
@RequiredArgsConstructor
public class AccountServiceImpl extends BaseServiceImpl<AccountMapper, AccountDO, AccountResp, AccountDetailResp, AccountQuery, AccountReq> implements AccountService {
    private final UserService userService;

    @Override
    @Cached(cacheType = CacheType.LOCAL, key = "#userId", name = CacheConstants.ENABLED_ACCOUNTS_KEY_PREFIX, localExpire = 300, expire = 300)
    public List<AccountResp> getAccountsByUserId(Long id, DisEnableStatusEnum status) {
        AccountQuery accountQuery = new AccountQuery();
        accountQuery.setStatus(status.getValue());
        accountQuery.setUserId(id);
        return list(accountQuery, null);
    }

    @Override
    @Cached(cacheType = CacheType.LOCAL, key = "#username", name = CacheConstants.ENABLED_ACCOUNTS_KEY_PREFIX, localExpire = 300, expire = 300)
    public UserDO getUserByAccountUsername(String username) {
        return baseMapper.lambdaQuery()
            .eq(AccountDO::getUsername, username)
            .eq(AccountDO::getStatus, DisEnableStatusEnum.ENABLE)
            .oneOpt()
            .map(account -> userService.getById(account.getUserId()))
            .orElse(null);
    }

    @Override
    public void updateHeaders(String headers, Long id) {
        baseMapper.lambdaUpdate().set(AccountDO::getHeaders, headers).eq(AccountDO::getId, id).update();
    }
}